@@ -2,19 +2,27 @@
 
 mod rule;
 
-pub use rule::*;
+#[doc(inline)]
+pub use rule::{AlternateTime, Julian0WithLeap, Julian1WithoutLeap, MonthWeekDay, RuleDay, TransitionRule};
 
-use crate::datetime::{days_since_unix_epoch, is_leap_year};
-use crate::error::*;
-use crate::utils::*;
-use crate::UtcDateTime;
+use crate::error::TzError;
+use crate::error::timezone::{LocalTimeTypeError, TimeZoneError};
+use crate::utils::{binary_search_leap_seconds, binary_search_transitions};
+
+#[cfg(feature = "alloc")]
+use crate::{
+    error::parse::TzStringError,
+    parse::{parse_posix_tz, parse_tz_file},
+};
 
-use core::cmp::Ordering;
 use core::fmt;
 use core::str;
 
 #[cfg(feature = "alloc")]
-use alloc::{vec, vec::Vec};
+use alloc::{boxed::Box, format, vec, vec::Vec};
+
+#[cfg(feature = "std")]
+use std::time::SystemTime;
 
 /// Transition of a TZif file
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
@@ -28,22 +36,19 @@ pub struct Transition {
 impl Transition {
     /// Construct a TZif file transition
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn new(unix_leap_time: i64, local_time_type_index: usize) -> Self {
+    pub const fn new(unix_leap_time: i64, local_time_type_index: usize) -> Self {
         Self { unix_leap_time, local_time_type_index }
     }
 
     /// Returns Unix leap time
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn unix_leap_time(&self) -> i64 {
+    pub const fn unix_leap_time(&self) -> i64 {
         self.unix_leap_time
     }
 
     /// Returns local time type index
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn local_time_type_index(&self) -> usize {
+    pub const fn local_time_type_index(&self) -> usize {
         self.local_time_type_index
     }
 }
@@ -60,27 +65,27 @@ pub struct LeapSecond {
 impl LeapSecond {
     /// Construct a TZif file leap second
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn new(unix_leap_time: i64, correction: i32) -> Self {
+    pub const fn new(unix_leap_time: i64, correction: i32) -> Self {
         Self { unix_leap_time, correction }
     }
 
     /// Returns Unix leap time
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn unix_leap_time(&self) -> i64 {
+    pub const fn unix_leap_time(&self) -> i64 {
         self.unix_leap_time
     }
 
     /// Returns leap second correction
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn correction(&self) -> i32 {
+    pub const fn correction(&self) -> i32 {
         self.correction
     }
 }
 
-/// ASCII-encoded fixed-capacity string, used for storing time zone designations
+/// ASCII-encoded fixed-capacity string, used for storing time zone designations.
+///
+/// POSIX only supports time zone designations with at least three characters,
+/// but this type is extended to also support military time zones like `"Z"`.
 #[derive(Copy, Clone, Eq, PartialEq)]
 struct TzAsciiStr {
     /// Length-prefixed string buffer
@@ -89,12 +94,11 @@ struct TzAsciiStr {
 
 impl TzAsciiStr {
     /// Construct a time zone designation string
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    fn new(input: &[u8]) -> Result<Self, LocalTimeTypeError> {
+    const fn new(input: &[u8]) -> Result<Self, LocalTimeTypeError> {
         let len = input.len();
 
-        if !(3 <= len && len <= 7) {
-            return Err(LocalTimeTypeError("time zone designation must have between 3 and 7 characters"));
+        if !(1 <= len && len <= 7) {
+            return Err(LocalTimeTypeError::InvalidTimeZoneDesignationLength);
         }
 
         let mut bytes = [0; 8];
@@ -105,7 +109,7 @@ impl TzAsciiStr {
             let b = input[i];
 
             if !matches!(b, b'0'..=b'9' | b'A'..=b'Z' | b'a'..=b'z' | b'+' | b'-') {
-                return Err(LocalTimeTypeError("invalid characters in time zone designation"));
+                return Err(LocalTimeTypeError::InvalidTimeZoneDesignationChar);
             }
 
             bytes[i + 1] = b;
@@ -118,30 +122,31 @@ impl TzAsciiStr {
 
     /// Returns time zone designation as a byte slice
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    fn as_bytes(&self) -> &[u8] {
+    const fn as_bytes(&self) -> &[u8] {
         match &self.bytes {
+            [1, head @ .., _, _, _, _, _, _] => head,
+            [2, head @ .., _, _, _, _, _] => head,
             [3, head @ .., _, _, _, _] => head,
             [4, head @ .., _, _, _] => head,
             [5, head @ .., _, _] => head,
             [6, head @ .., _] => head,
             [7, head @ ..] => head,
-            _ => const_panic!(), // unreachable
+            _ => unreachable!(),
         }
     }
 
     /// Returns time zone designation as a string
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    fn as_str(&self) -> &str {
-        // SAFETY: ASCII is valid UTF-8
-        unsafe { str::from_utf8_unchecked(self.as_bytes()) }
+    const fn as_str(&self) -> &str {
+        match str::from_utf8(self.as_bytes()) {
+            Ok(s) => s,
+            Err(_) => panic!("unreachable code: ASCII is valid UTF-8"),
+        }
     }
 
     /// Check if two time zone designations are equal
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    fn equal(&self, other: &Self) -> bool {
+    const fn equal(&self, other: &Self) -> bool {
         u64::from_ne_bytes(self.bytes) == u64::from_ne_bytes(other.bytes)
     }
 }
@@ -165,10 +170,9 @@ pub struct LocalTimeType {
 
 impl LocalTimeType {
     /// Construct a local time type
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn new(ut_offset: i32, is_dst: bool, time_zone_designation: Option<&[u8]>) -> Result<Self, LocalTimeTypeError> {
+    pub const fn new(ut_offset: i32, is_dst: bool, time_zone_designation: Option<&[u8]>) -> Result<Self, LocalTimeTypeError> {
         if ut_offset == i32::MIN {
-            return Err(LocalTimeTypeError("invalid UTC offset"));
+            return Err(LocalTimeTypeError::InvalidUtcOffset);
         }
 
         let time_zone_designation = match time_zone_designation {
@@ -190,10 +194,9 @@ impl LocalTimeType {
 
     /// Construct a local time type with the specified UTC offset in seconds
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn with_ut_offset(ut_offset: i32) -> Result<Self, LocalTimeTypeError> {
+    pub const fn with_ut_offset(ut_offset: i32) -> Result<Self, LocalTimeTypeError> {
         if ut_offset == i32::MIN {
-            return Err(LocalTimeTypeError("invalid UTC offset"));
+            return Err(LocalTimeTypeError::InvalidUtcOffset);
         }
 
         Ok(Self { ut_offset, is_dst: false, time_zone_designation: None })
@@ -201,22 +204,19 @@ impl LocalTimeType {
 
     /// Returns offset from UTC in seconds
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn ut_offset(&self) -> i32 {
+    pub const fn ut_offset(&self) -> i32 {
         self.ut_offset
     }
 
     /// Returns daylight saving time indicator
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn is_dst(&self) -> bool {
+    pub const fn is_dst(&self) -> bool {
         self.is_dst
     }
 
     /// Returns time zone designation
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn time_zone_designation(&self) -> &str {
+    pub const fn time_zone_designation(&self) -> &str {
         match &self.time_zone_designation {
             Some(s) => s.as_str(),
             None => "",
@@ -225,8 +225,7 @@ impl LocalTimeType {
 
     /// Check if two local time types are equal
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    fn equal(&self, other: &Self) -> bool {
+    const fn equal(&self, other: &Self) -> bool {
         self.ut_offset == other.ut_offset
             && self.is_dst == other.is_dst
             && match (&self.time_zone_designation, &other.time_zone_designation) {
@@ -237,21 +236,6 @@ impl LocalTimeType {
     }
 }
 
-/// Time zone
-#[cfg(feature = "alloc")]
-#[cfg_attr(docsrs, doc(cfg(feature = "alloc")))]
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub struct TimeZone {
-    /// List of transitions
-    transitions: Vec<Transition>,
-    /// List of local time types (cannot be empty)
-    local_time_types: Vec<LocalTimeType>,
-    /// List of leap seconds
-    leap_seconds: Vec<LeapSecond>,
-    /// Extra transition rule applicable after the last transition
-    extra_rule: Option<TransitionRule>,
-}
-
 /// Reference to a time zone
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub struct TimeZoneRef<'a> {
@@ -267,13 +251,12 @@ pub struct TimeZoneRef<'a> {
 
 impl<'a> TimeZoneRef<'a> {
     /// Construct a time zone reference
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn new(
+    pub const fn new(
         transitions: &'a [Transition],
         local_time_types: &'a [LocalTimeType],
         leap_seconds: &'a [LeapSecond],
         extra_rule: &'a Option<TransitionRule>,
-    ) -> Result<Self, TimeZoneError> {
+    ) -> Result<Self, TzError> {
         let time_zone_ref = Self::new_unchecked(transitions, local_time_types, leap_seconds, extra_rule);
 
         if let Err(error) = time_zone_ref.check_inputs() {
@@ -285,43 +268,36 @@ impl<'a> TimeZoneRef<'a> {
 
     /// Construct the time zone reference associated to UTC
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn utc() -> Self {
-        const UTC: LocalTimeType = LocalTimeType::utc();
-        Self { transitions: &[], local_time_types: &[UTC], leap_seconds: &[], extra_rule: &None }
+    pub const fn utc() -> Self {
+        Self { transitions: &[], local_time_types: &[const { LocalTimeType::utc() }], leap_seconds: &[], extra_rule: &None }
     }
 
     /// Returns list of transitions
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn transitions(&self) -> &'a [Transition] {
+    pub const fn transitions(&self) -> &'a [Transition] {
         self.transitions
     }
 
     /// Returns list of local time types
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn local_time_types(&self) -> &'a [LocalTimeType] {
+    pub const fn local_time_types(&self) -> &'a [LocalTimeType] {
         self.local_time_types
     }
 
     /// Returns list of leap seconds
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn leap_seconds(&self) -> &'a [LeapSecond] {
+    pub const fn leap_seconds(&self) -> &'a [LeapSecond] {
         self.leap_seconds
     }
 
     /// Returns extra transition rule applicable after the last transition
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn extra_rule(&self) -> &'a Option<TransitionRule> {
+    pub const fn extra_rule(&self) -> &'a Option<TransitionRule> {
         self.extra_rule
     }
 
     /// Find the local time type associated to the time zone at the specified Unix time in seconds
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub fn find_local_time_type(&self, unix_time: i64) -> Result<&'a LocalTimeType, FindLocalTimeTypeError> {
+    pub const fn find_local_time_type(&self, unix_time: i64) -> Result<&'a LocalTimeType, TzError> {
         let extra_rule = match self.transitions {
             [] => match self.extra_rule {
                 Some(extra_rule) => extra_rule,
@@ -330,13 +306,13 @@ impl<'a> TimeZoneRef<'a> {
             [.., last_transition] => {
                 let unix_leap_time = match self.unix_time_to_unix_leap_time(unix_time) {
                     Ok(unix_leap_time) => unix_leap_time,
-                    Err(OutOfRangeError(error)) => return Err(FindLocalTimeTypeError(error)),
+                    Err(error) => return Err(error),
                 };
 
                 if unix_leap_time >= last_transition.unix_leap_time {
                     match self.extra_rule {
                         Some(extra_rule) => extra_rule,
-                        None => return Err(FindLocalTimeTypeError("no local time type is available for the specified timestamp")),
+                        None => return Err(TzError::NoAvailableLocalTimeType),
                     }
                 } else {
                     let index = match binary_search_transitions(self.transitions, unix_leap_time) {
@@ -350,16 +326,12 @@ impl<'a> TimeZoneRef<'a> {
             }
         };
 
-        match extra_rule.find_local_time_type(unix_time) {
-            Ok(local_time_type) => Ok(local_time_type),
-            Err(OutOfRangeError(error)) => Err(FindLocalTimeTypeError(error)),
-        }
+        extra_rule.find_local_time_type(unix_time)
     }
 
     /// Construct a reference to a time zone
     #[inline]
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    fn new_unchecked(
+    const fn new_unchecked(
         transitions: &'a [Transition],
         local_time_types: &'a [LocalTimeType],
         leap_seconds: &'a [LeapSecond],
@@ -369,25 +341,24 @@ impl<'a> TimeZoneRef<'a> {
     }
 
     /// Check time zone inputs
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    fn check_inputs(&self) -> Result<(), TimeZoneError> {
+    const fn check_inputs(&self) -> Result<(), TzError> {
         use crate::constants::*;
 
         // Check local time types
         let local_time_types_size = self.local_time_types.len();
         if local_time_types_size == 0 {
-            return Err(TimeZoneError("list of local time types must not be empty"));
+            return Err(TzError::TimeZone(TimeZoneError::NoLocalTimeType));
         }
 
         // Check transitions
         let mut i_transition = 0;
         while i_transition < self.transitions.len() {
             if self.transitions[i_transition].local_time_type_index >= local_time_types_size {
-                return Err(TimeZoneError("invalid local time type index"));
+                return Err(TzError::TimeZone(TimeZoneError::InvalidLocalTimeTypeIndex));
             }
 
             if i_transition + 1 < self.transitions.len() && self.transitions[i_transition].unix_leap_time >= self.transitions[i_transition + 1].unix_leap_time {
-                return Err(TimeZoneError("invalid transition"));
+                return Err(TzError::TimeZone(TimeZoneError::InvalidTransition));
             }
 
             i_transition += 1;
@@ -395,7 +366,7 @@ impl<'a> TimeZoneRef<'a> {
 
         // Check leap seconds
         if !(self.leap_seconds.is_empty() || self.leap_seconds[0].unix_leap_time >= 0 && self.leap_seconds[0].correction.saturating_abs() == 1) {
-            return Err(TimeZoneError("invalid leap second"));
+            return Err(TzError::TimeZone(TimeZoneError::InvalidLeapSecond));
         }
 
         let min_interval = SECONDS_PER_28_DAYS - 1;
@@ -410,7 +381,7 @@ impl<'a> TimeZoneRef<'a> {
                 let abs_diff_correction = x1.correction.saturating_sub(x0.correction).saturating_abs();
 
                 if !(diff_unix_leap_time >= min_interval && abs_diff_correction == 1) {
-                    return Err(TimeZoneError("invalid leap second"));
+                    return Err(TzError::TimeZone(TimeZoneError::InvalidLeapSecond));
                 }
             }
             i_leap_second += 1;
@@ -422,16 +393,16 @@ impl<'a> TimeZoneRef<'a> {
 
             let unix_time = match self.unix_leap_time_to_unix_time(last_transition.unix_leap_time) {
                 Ok(unix_time) => unix_time,
-                Err(OutOfRangeError(error)) => return Err(TimeZoneError(error)),
+                Err(error) => return Err(error),
             };
 
             let rule_local_time_type = match extra_rule.find_local_time_type(unix_time) {
                 Ok(rule_local_time_type) => rule_local_time_type,
-                Err(OutOfRangeError(error)) => return Err(TimeZoneError(error)),
+                Err(error) => return Err(error),
             };
 
             if !last_local_time_type.equal(rule_local_time_type) {
-                return Err(TimeZoneError("extra transition rule is inconsistent with the last transition"));
+                return Err(TzError::TimeZone(TimeZoneError::InconsistentExtraRule));
             }
         }
 
@@ -439,8 +410,7 @@ impl<'a> TimeZoneRef<'a> {
     }
 
     /// Convert Unix time to Unix leap time, from the list of leap seconds in a time zone
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub(crate) fn unix_time_to_unix_leap_time(&self, unix_time: i64) -> Result<i64, OutOfRangeError> {
+    pub(crate) const fn unix_time_to_unix_leap_time(&self, unix_time: i64) -> Result<i64, TzError> {
         let mut unix_leap_time = unix_time;
 
         let mut i = 0;
@@ -453,7 +423,7 @@ impl<'a> TimeZoneRef<'a> {
 
             unix_leap_time = match unix_time.checked_add(leap_second.correction as i64) {
                 Some(unix_leap_time) => unix_leap_time,
-                None => return Err(OutOfRangeError("out of range operation")),
+                None => return Err(TzError::OutOfRange),
             };
 
             i += 1;
@@ -463,10 +433,9 @@ impl<'a> TimeZoneRef<'a> {
     }
 
     /// Convert Unix leap time to Unix time, from the list of leap seconds in a time zone
-    #[cfg_attr(feature = "const", const_fn::const_fn)]
-    pub(crate) fn unix_leap_time_to_unix_time(&self, unix_leap_time: i64) -> Result<i64, OutOfRangeError> {
+    pub(crate) const fn unix_leap_time_to_unix_time(&self, unix_leap_time: i64) -> Result<i64, TzError> {
         if unix_leap_time == i64::MIN {
-            return Err(OutOfRangeError("out of range operation"));
+            return Err(TzError::OutOfRange);
         }
 
         let index = match binary_search_leap_seconds(self.leap_seconds, unix_leap_time - 1) {
@@ -478,11 +447,25 @@ impl<'a> TimeZoneRef<'a> {
 
         match unix_leap_time.checked_sub(correction as i64) {
             Some(unix_time) => Ok(unix_time),
-            None => Err(OutOfRangeError("out of range operation")),
+            None => Err(TzError::OutOfRange),
         }
     }
 }
 
+/// Time zone
+#[cfg(feature = "alloc")]
+#[derive(Debug, Clone, Eq, PartialEq)]
+pub struct TimeZone {
+    /// List of transitions
+    transitions: Vec<Transition>,
+    /// List of local time types (cannot be empty)
+    local_time_types: Vec<LocalTimeType>,
+    /// List of leap seconds
+    leap_seconds: Vec<LeapSecond>,
+    /// Extra transition rule applicable after the last transition
+    extra_rule: Option<TransitionRule>,
+}
+
 #[cfg(feature = "alloc")]
 impl TimeZone {
     /// Construct a time zone
@@ -491,14 +474,14 @@ impl TimeZone {
         local_time_types: Vec<LocalTimeType>,
         leap_seconds: Vec<LeapSecond>,
         extra_rule: Option<TransitionRule>,
-    ) -> Result<Self, TimeZoneError> {
+    ) -> Result<Self, TzError> {
         TimeZoneRef::new_unchecked(&transitions, &local_time_types, &leap_seconds, &extra_rule).check_inputs()?;
         Ok(Self { transitions, local_time_types, leap_seconds, extra_rule })
     }
 
     /// Returns a reference to the time zone
     #[inline]
-    pub fn as_ref(&self) -> TimeZoneRef {
+    pub fn as_ref(&self) -> TimeZoneRef<'_> {
         TimeZoneRef::new_unchecked(&self.transitions, &self.local_time_types, &self.leap_seconds, &self.extra_rule)
     }
 
@@ -514,59 +497,102 @@ impl TimeZone {
         Ok(Self { transitions: Vec::new(), local_time_types: vec![LocalTimeType::with_ut_offset(ut_offset)?], leap_seconds: Vec::new(), extra_rule: None })
     }
 
+    /// Find the local time type associated to the time zone at the specified Unix time in seconds
+    pub fn find_local_time_type(&self, unix_time: i64) -> Result<&LocalTimeType, TzError> {
+        self.as_ref().find_local_time_type(unix_time)
+    }
+
+    /// Construct a time zone from the contents of a time zone file
+    pub fn from_tz_data(bytes: &[u8]) -> Result<Self, TzError> {
+        parse_tz_file(bytes)
+    }
+
     /// Returns local time zone.
     ///
     /// This method in not supported on non-UNIX platforms, and returns the UTC time zone instead.
     ///
     #[cfg(feature = "std")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "std")))]
-    pub fn local() -> Result<Self, TzError> {
-        #[cfg(not(unix))]
-        let local_time_zone = Self::utc();
-
-        #[cfg(unix)]
-        let local_time_zone = Self::from_posix_tz("localtime")?;
-
-        Ok(local_time_zone)
-    }
-
-    /// Construct a time zone from the contents of a time zone file
-    #[cfg(feature = "std")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "std")))]
-    pub fn from_tz_data(bytes: &[u8]) -> Result<Self, TzError> {
-        crate::parse::parse_tz_file(bytes)
+    pub fn local() -> Result<Self, crate::Error> {
+        TimeZoneSettings::DEFAULT.parse_local()
     }
 
     /// Construct a time zone from a POSIX TZ string, as described in [the POSIX documentation of the `TZ` environment variable](https://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap08.html).
     #[cfg(feature = "std")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "std")))]
-    pub fn from_posix_tz(tz_string: &str) -> Result<Self, TzError> {
-        use crate::parse::*;
+    pub fn from_posix_tz(tz_string: &str) -> Result<Self, crate::Error> {
+        TimeZoneSettings::DEFAULT.parse_posix_tz(tz_string)
+    }
 
-        use std::fs::{self, File};
-        use std::io::{self, Read};
+    /// Find the current local time type associated to the time zone
+    #[cfg(feature = "std")]
+    pub fn find_current_local_time_type(&self) -> Result<&LocalTimeType, TzError> {
+        self.find_local_time_type(crate::utils::system_time::unix_time(SystemTime::now()))
+    }
+}
 
+/// Read file function type alias
+#[cfg(feature = "alloc")]
+type ReadFileFn = fn(path: &str) -> Result<Vec<u8>, Box<dyn core::error::Error + Send + Sync + 'static>>;
+
+/// Time zone settings
+#[cfg(feature = "alloc")]
+#[derive(Debug)]
+pub struct TimeZoneSettings<'a> {
+    /// Possible system timezone directories
+    directories: &'a [&'a str],
+    /// Read file function
+    read_file_fn: ReadFileFn,
+}
+
+#[cfg(feature = "alloc")]
+impl<'a> TimeZoneSettings<'a> {
+    /// Default possible system timezone directories
+    pub const DEFAULT_DIRECTORIES: &'static [&'static str] = &["/usr/share/zoneinfo", "/share/zoneinfo", "/etc/zoneinfo"];
+
+    /// Default read file function
+    #[cfg(feature = "std")]
+    pub const DEFAULT_READ_FILE_FN: ReadFileFn = |path| Ok(std::fs::read(path)?);
+
+    /// Default time zone settings
+    #[cfg(feature = "std")]
+    pub const DEFAULT: TimeZoneSettings<'static> = TimeZoneSettings { directories: Self::DEFAULT_DIRECTORIES, read_file_fn: Self::DEFAULT_READ_FILE_FN };
+
+    /// Construct time zone settings
+    pub const fn new(directories: &'a [&'a str], read_file_fn: ReadFileFn) -> TimeZoneSettings<'a> {
+        Self { directories, read_file_fn }
+    }
+
+    /// Returns local time zone using current settings.
+    ///
+    /// This method in not supported on non-UNIX platforms, and returns the UTC time zone instead.
+    ///
+    pub fn parse_local(&self) -> Result<TimeZone, crate::Error> {
+        #[cfg(not(unix))]
+        let local_time_zone = TimeZone::utc();
+
+        #[cfg(unix)]
+        let local_time_zone = self.parse_posix_tz("localtime")?;
+
+        Ok(local_time_zone)
+    }
+
+    /// Construct a time zone from a POSIX TZ string using current settings,
+    /// as described in [the POSIX documentation of the `TZ` environment variable](https://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap08.html).
+    pub fn parse_posix_tz(&self, tz_string: &str) -> Result<TimeZone, crate::Error> {
         if tz_string.is_empty() {
-            return Err(TzError::TzStringError(TzStringError::InvalidTzString("empty TZ string")));
+            return Err(TzStringError::Empty.into());
         }
 
         if tz_string == "localtime" {
-            return parse_tz_file(&fs::read("/etc/localtime")?);
+            return Ok(parse_tz_file(&(self.read_file_fn)("/etc/localtime").map_err(crate::Error::Io)?)?);
         }
 
-        let read = |mut file: File| -> io::Result<_> {
-            let mut bytes = Vec::new();
-            file.read_to_end(&mut bytes)?;
-            Ok(bytes)
-        };
-
         let mut chars = tz_string.chars();
         if chars.next() == Some(':') {
-            return parse_tz_file(&read(get_tz_file(chars.as_str())?)?);
+            return Ok(parse_tz_file(&self.read_tz_file(chars.as_str())?)?);
         }
 
-        match get_tz_file(tz_string) {
-            Ok(file) => parse_tz_file(&read(file)?),
+        match self.read_tz_file(tz_string) {
+            Ok(bytes) => Ok(parse_tz_file(&bytes)?),
             Err(_) => {
                 let tz_string = tz_string.trim_matches(|c: char| c.is_ascii_whitespace());
 
@@ -583,49 +609,52 @@ impl TimeZone {
         }
     }
 
-    /// Find the current local time type associated to the time zone
-    #[cfg(feature = "std")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "std")))]
-    pub fn find_current_local_time_type(&self) -> Result<&LocalTimeType, TzError> {
-        use core::convert::TryInto;
-        use std::time::SystemTime;
+    /// Read the TZif file corresponding to a TZ string using current settings
+    fn read_tz_file(&self, tz_string: &str) -> Result<Vec<u8>, crate::Error> {
+        let read_file_fn = |path: &str| (self.read_file_fn)(path).map_err(crate::Error::Io);
 
-        Ok(self.find_local_time_type(SystemTime::now().duration_since(SystemTime::UNIX_EPOCH)?.as_secs().try_into()?)?)
-    }
+        // Don't check system timezone directories on non-UNIX platforms
+        #[cfg(not(unix))]
+        return Ok(read_file_fn(tz_string)?);
 
-    /// Find the local time type associated to the time zone at the specified Unix time in seconds
-    pub fn find_local_time_type(&self, unix_time: i64) -> Result<&LocalTimeType, FindLocalTimeTypeError> {
-        self.as_ref().find_local_time_type(unix_time)
+        #[cfg(unix)]
+        if tz_string.starts_with('/') {
+            Ok(read_file_fn(tz_string)?)
+        } else {
+            self.directories
+                .iter()
+                .find_map(|folder| read_file_fn(&format!("{folder}/{tz_string}")).ok())
+                .ok_or_else(|| crate::Error::Io("file was not found".into()))
+        }
     }
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
-    use crate::Result;
 
     #[test]
-    fn test_tz_ascii_str() -> Result<()> {
-        assert!(matches!(TzAsciiStr::new(b""), Err(LocalTimeTypeError(_))));
-        assert!(matches!(TzAsciiStr::new(b"1"), Err(LocalTimeTypeError(_))));
-        assert!(matches!(TzAsciiStr::new(b"12"), Err(LocalTimeTypeError(_))));
+    fn test_tz_ascii_str() -> Result<(), TzError> {
+        assert!(matches!(TzAsciiStr::new(b""), Err(LocalTimeTypeError::InvalidTimeZoneDesignationLength)));
+        assert_eq!(TzAsciiStr::new(b"1")?.as_bytes(), b"1");
+        assert_eq!(TzAsciiStr::new(b"12")?.as_bytes(), b"12");
         assert_eq!(TzAsciiStr::new(b"123")?.as_bytes(), b"123");
         assert_eq!(TzAsciiStr::new(b"1234")?.as_bytes(), b"1234");
         assert_eq!(TzAsciiStr::new(b"12345")?.as_bytes(), b"12345");
         assert_eq!(TzAsciiStr::new(b"123456")?.as_bytes(), b"123456");
         assert_eq!(TzAsciiStr::new(b"1234567")?.as_bytes(), b"1234567");
-        assert!(matches!(TzAsciiStr::new(b"12345678"), Err(LocalTimeTypeError(_))));
-        assert!(matches!(TzAsciiStr::new(b"123456789"), Err(LocalTimeTypeError(_))));
-        assert!(matches!(TzAsciiStr::new(b"1234567890"), Err(LocalTimeTypeError(_))));
+        assert!(matches!(TzAsciiStr::new(b"12345678"), Err(LocalTimeTypeError::InvalidTimeZoneDesignationLength)));
+        assert!(matches!(TzAsciiStr::new(b"123456789"), Err(LocalTimeTypeError::InvalidTimeZoneDesignationLength)));
+        assert!(matches!(TzAsciiStr::new(b"1234567890"), Err(LocalTimeTypeError::InvalidTimeZoneDesignationLength)));
 
-        assert!(matches!(TzAsciiStr::new(b"123\0\0\0"), Err(LocalTimeTypeError(_))));
+        assert!(matches!(TzAsciiStr::new(b"123\0\0\0"), Err(LocalTimeTypeError::InvalidTimeZoneDesignationChar)));
 
         Ok(())
     }
 
     #[cfg(feature = "alloc")]
     #[test]
-    fn test_time_zone() -> Result<()> {
+    fn test_time_zone() -> Result<(), TzError> {
         let utc = LocalTimeType::utc();
         let cet = LocalTimeType::with_ut_offset(3600)?;
 
@@ -641,7 +670,7 @@ mod test {
         assert_eq!(*time_zone_2.find_local_time_type(0)?, cet);
 
         assert_eq!(*time_zone_3.find_local_time_type(-1)?, utc);
-        assert!(matches!(time_zone_3.find_local_time_type(0), Err(FindLocalTimeTypeError(_))));
+        assert!(matches!(time_zone_3.find_local_time_type(0), Err(TzError::NoAvailableLocalTimeType)));
 
         assert_eq!(*time_zone_4.find_local_time_type(-1)?, utc);
         assert_eq!(*time_zone_4.find_local_time_type(0)?, cet);
@@ -654,7 +683,7 @@ mod test {
 
     #[cfg(feature = "std")]
     #[test]
-    fn test_time_zone_from_posix_tz() -> Result<()> {
+    fn test_time_zone_from_posix_tz() -> Result<(), crate::Error> {
         #[cfg(unix)]
         {
             let time_zone_local = TimeZone::local()?;
@@ -666,7 +695,7 @@ mod test {
             assert_eq!(time_zone_local, time_zone_local_2);
             assert_eq!(time_zone_local, time_zone_local_3);
 
-            assert!(matches!(time_zone_local.find_current_local_time_type(), Ok(_) | Err(TzError::FindLocalTimeTypeError(_))));
+            assert!(matches!(time_zone_local.find_current_local_time_type(), Ok(_) | Err(TzError::NoAvailableLocalTimeType)));
 
             let time_zone_utc = TimeZone::from_posix_tz("UTC")?;
             assert_eq!(time_zone_utc.find_local_time_type(0)?.ut_offset(), 0);
@@ -680,7 +709,7 @@ mod test {
 
     #[cfg(feature = "alloc")]
     #[test]
-    fn test_leap_seconds() -> Result<()> {
+    fn test_leap_seconds() -> Result<(), TzError> {
         let time_zone = TimeZone::new(
             vec![],
             vec![LocalTimeType::new(0, false, Some(b"UTC"))?],
@@ -732,7 +761,7 @@ mod test {
 
     #[cfg(feature = "alloc")]
     #[test]
-    fn test_leap_seconds_overflow() -> Result<()> {
+    fn test_leap_seconds_overflow() -> Result<(), TzError> {
         let time_zone_err = TimeZone::new(
             vec![Transition::new(i64::MIN, 0)],
             vec![LocalTimeType::utc()],
@@ -742,7 +771,7 @@ mod test {
         assert!(time_zone_err.is_err());
 
         let time_zone = TimeZone::new(vec![Transition::new(i64::MAX, 0)], vec![LocalTimeType::utc()], vec![LeapSecond::new(0, 1)], None)?;
-        assert!(matches!(time_zone.find_local_time_type(i64::MAX), Err(FindLocalTimeTypeError(_))));
+        assert!(matches!(time_zone.find_local_time_type(i64::MAX), Err(TzError::OutOfRange)));
 
         Ok(())
     }
